@@ -34,8 +34,7 @@ _default_options = {}
 
 
 class StringIO(list):
-    """Unicode aware append-only version of StringIO.
-    """
+    # Unicode aware append-only version of StringIO.
     write = list.append
 
     def __init__(self, value=None):
@@ -49,29 +48,31 @@ class StringIO(list):
 
 @implementer(IPageTemplateSubclassing)
 class PageTemplate(object):
-    """Page Templates using TAL, TALES, and METAL.
+    """
+    Page Templates using TAL, TALES, and METAL.
 
-    Subclassing
-    -----------
+    **Subclassing**
+
+    This class implements :class:`~zope.pagetemplate.interfaces.IPageTemplateSubclassing`.
 
     The following methods have certain internal responsibilities.
 
-    pt_getContext(**keywords)
+    ``pt_getContext(**keywords)``
         Should ignore keyword arguments that it doesn't care about,
         and construct the namespace passed to the TALES expression
         engine.  This method is free to use the keyword arguments it
         receives.
 
-    pt_render(namespace, source=False, sourceAnnotations=False, showtal=False)
+    ``pt_render(namespace, source=False, sourceAnnotations=False, showtal=False)``
         Responsible the TAL interpreter to perform the rendering.  The
         namespace argument is a mapping which defines the top-level
         namespaces passed to the TALES expression engine.
 
-    __call__(*args, **keywords)
+    ``__call__(*args, **keywords)``
         Calls pt_getContext() to construct the top-level namespace
         passed to the TALES expression engine, then calls pt_render()
         to perform the rendering.
-    """
+    """  # noqa: E501 line too long
 
     _error_start = '<!-- Page Template Diagnostics'
     _error_end = '-->'
@@ -85,12 +86,11 @@ class PageTemplate(object):
     _v_program = None
     _text = ''
 
+    @property
     def macros(self):
         self._cook_check()
         return self._v_macros
 
-    macros = property(macros)
-
     def pt_edit(self, text, content_type):
         if content_type:
             self.content_type = str(content_type)
@@ -123,7 +123,7 @@ class PageTemplate(object):
 
         __traceback_supplement__ = (
             PageTemplateTracebackSupplement, self, namespace
-            )
+        )
 
         if self._v_errors:
             raise PTRuntimeError(str(self._v_errors))
@@ -133,7 +133,7 @@ class PageTemplate(object):
         return self._v_program(
             context, self._v_macros, tal=not source, showtal=showtal,
             strictinsert=0, sourceAnnotations=sourceAnnotations
-            )
+        )
 
     def pt_errors(self, namespace, check_macro_expansion=True):
         self._cook_check()
@@ -143,15 +143,27 @@ class PageTemplate(object):
         if check_macro_expansion:
             try:
                 self.pt_render(namespace, source=1)
-            except:
-                return ('Macro expansion failed', '%s: %s' % sys.exc_info()[:2])
+            except Exception:
+                return (
+                    'Macro expansion failed', '%s: %s' % sys.exc_info()[:2])
 
     def _convert(self, string, text):
         """Adjust the string type to the type of text"""
-        if isinstance(text, six.binary_type):
+        if isinstance(
+                text,
+                six.binary_type) and not isinstance(
+                string,
+                six.binary_type):
             return string.encode('utf-8')
-        else:
-            return string
+
+        if isinstance(
+                text,
+                six.text_type) and not isinstance(
+                string,
+                six.text_type):
+            return string.decode('utf-8')
+
+        return string
 
     def write(self, text):
         # We accept both, since the text can either come from a file (and the
@@ -179,6 +191,7 @@ class PageTemplate(object):
     def read(self, request=None):
         """Gets the source, sometimes with macros expanded."""
         self._cook_check()
+
         def bs(s):
             """Bytes or str"""
             return self._convert(s, self._text)
@@ -191,14 +204,14 @@ class PageTemplate(object):
                 # this point, since we are not evaluating the template.
                 context = self.pt_getContext(self, request)
                 return self.pt_render(context, source=1)
-            except:
+            except BaseException:
                 return (bs('%s\n Macro expansion failed\n %s\n-->\n' %
-                           (self._error_start, "%s: %s" % sys.exc_info()[:2])) +
-                        self._text)
+                           (self._error_start, "%s: %s" %
+                            sys.exc_info()[:2])) + self._text)
 
         return bs('%s\n %s\n-->\n' % (self._error_start,
                                       '\n'.join(self._v_errors))) + \
-               self._text
+            self._text
 
     def pt_source_file(self):
         """To be overridden."""
@@ -222,15 +235,18 @@ class PageTemplate(object):
         try:
             engine = queryUtility(
                 IPageTemplateEngine, default=PageTemplateEngine
-                )
+            )
             self._v_program, self._v_macros = engine.cook(
                 source_file, self._text, pt_engine, self.content_type)
-        except:
+        except BaseException:
             etype, e = sys.exc_info()[:2]
-            self._v_errors = [
-                "Compilation failed",
-                "%s.%s: %s" % (etype.__module__, etype.__name__, e)
+            try:
+                self._v_errors = [
+                    "Compilation failed",
+                    "%s.%s: %s" % (etype.__module__, etype.__name__, e)
                 ]
+            finally:
+                del e
 
         self._v_cooked = 1
 
@@ -243,8 +259,12 @@ class PTRuntimeError(RuntimeError):
 @implementer(IPageTemplateProgram)
 @provider(IPageTemplateEngine)
 class PageTemplateEngine(object):
-    """Page template engine that uses the TAL interpreter to render."""
+    """
+    Page template engine that uses the TAL interpreter to render.
 
+    This class implements
+    :class:`zope.pagetemplate.interfaces.IPageTemplateProgram`.
+    """
 
     def __init__(self, program):
         self.program = program
@@ -254,7 +274,7 @@ class PageTemplateEngine(object):
         interpreter = TALInterpreter(
             self.program, macros, context,
             stream=output, **options
-            )
+        )
         interpreter()
         return output.getvalue()
 
@@ -273,7 +293,7 @@ class PageTemplateEngine(object):
         return cls(program), macros
 
 
-#@implementer(ITracebackSupplement)
+# @implementer(ITracebackSupplement)
 class PageTemplateTracebackSupplement(object):
 
     def __init__(self, pt, namespace):
